<?php
/**
 * Je l'ai mis en classe abstraite car il faudra toujours au moins définir listeOption.
 * 
 * Je conseille de faire des getteur pour chaque option en la définissant avec la méthode get($nomOption) pour pouvoir utiliser l'autocompletion
 * 
 * Il est possible de faire un traitement spécifique lors de la mise à jour et lors de l'affichage si les traitements par défaut ne vous conviennent pas.
 * Pour cela, il faut redéfinir les méthodes :
 * 	- estSpecifique() : qui permettra de savoir qu'il faudra effectuer un traitement spécifique.
 * 						Attention à ne pas oublier de séparer les cas traitements et affichage si besoin grace au second parametre
 *  - affichageSpecifique() et majSpecifique() pour effectuer les traitements spécifiques lors les mise à jour et de l'affichage.
 *  					(Il est à votre charge de tester quelle traitement effectuer en fonction du paramètre avec un switch par exemple)
 *  
 *  
 * 
 * @author superbaillot
 * 
 * Copyright 2012-2013 Superbaillot
 * This file is part of Aeres.
 *
 *  Aeres is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Aeres is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *
 */

abstract class Configuration
{
	const STRING = 1;
	const FLOAT = 2;
	const BOOLEAN = 3;
	const LANGUE = 4;
	const ARRAY2 = 5;
	
	
	protected $listeOption; //= array("nom"=> array(init, type, label , description, message d'erreur)); 
	
	protected $pseudo = "";								//pseudo affiché dans le titre
	protected $passCrypte = "";							//mot de passe hashé
	protected $fichierConf = "";						//chemin vers le fichier de configuration
	protected $nomSession = "connexionSuperbaillot";				//racine du nom de la session
	
	public function getPseudo(){return $this->pseudo;}
	public function setPseudo($pseudo){$this->pseudo = $pseudo;}

	public function getPassCrypte(){return $this->passCrypte;}
	public function setPassCrypte($passCrypte){$this->passCrypte = $passCrypte;}

	public function getFichierConf(){return $this->fichierConf;}
	public function setFichierConf($fichierConf){$this->fichierConf = $fichierConf;}

	public function getNomSession(){return $this->nomSession;}
	public function setNomSession($nomSession){$this->nomSession = $nomSession;}

	public function getListeOption(){return $this->listeOption;}
	
	public function get($nomOption)
	{
		if(isset($this->listeOption[$nomOption]))
		{
			return $this->listeOption[$nomOption][0];
		}
		return false;
	}

	
	/**
	 * méthode qui définie quel formulaire on utilise.
	 * Il faut en effet utiliser un formulaire étendu pour personnaliser certaines option comme le timezone qui est la fusion de 2 champs
	 * @return FormulaireConfiguration
	 */
	public function getFormulaireConfiguration()
	{
		return new FormulaireConfiguration($this->listeOption);
	}
	
	
	/**
	 * définie si un champs ne se comporte pas de manière générique pour le traitement ou pour l'affichage
	 * @param $nom					String	: nom du champs à personnaliser
	 * @param $afficheSpecifique	boolean	: définie si c'est l'affichage ou le traitement de mise à jour qui est personnalisé
	 * @return boolean
	 */
	public function estSpecifique($nom, $afficheSpecifique=false)
	{
		return false;
	}
	
	
	/**
	 * méthode qui définie l'affichage spéficique pour certains champs (qui n'utilisent pas la présentation par défaut de son type)
	 * @param $nom	String	: nom du champs à personnaliser
	 * @return BaliseHTML
	 */
	public function affichageSpecifique($nom)
	{
		return new BaliseVide();
	}
	
	
	/**
	 * définie les actions personnalisées lors d'une mise à jour
	 * @param $nom	String	: nom du champs à personnaliser
	 * @return boolean	: vrai si il y a eu une modification et qu'il faut sauvegarder la configuration
	 */
	public function majSpecifique($nom, $form)
	{
		return false;
	}
	

	/**
	 * fonction qui met à jour la configuration
	 * @return String : message d'info
	 */
	public function maj()
	{
		$msg = "";
		try
		{
			$form = $this->getFormulaireConfiguration();

			$ecrireFichier = false;

			//si l'utilisateur a saisi un champ pour changer le mot de passe
			if($form->getPass() != "" || $form->getPassAncien() != "")
			{
				//on vérifie que l'ancien mot de passe est correct
				if(hashSB($form->getPassAncien()) != $this->passCrypte && $this->passCrypte != "" && $form->getPassAncien() != "")
				{
					$msg .= $GLOBALS["langue"]->get("CONFIG_OLD_PSWRD_INCORRECT") . "<br>\n";
				}
				//on enregistre le nouveau mot de passe
				else
				{
					$ecrireFichier = true;
					$this->passCrypte = hashSB($form->getPass());
					
					//on vérifie la taille du mot de passe (minimum syndical de sécurité, et encore...).
					//MAJ : ceci est juste un conseil, plus une obligation, la sécurité doit être voulue.
					if(strlen($form->getPass()) < 6)
					{
						$msg .= "<br>\n" . $GLOBALS["langue"]->get("CONFIG_PASS_COURT") . "<br>\n";
					}
				}
			}

			//changement de pseudo
			if($form->getPseudo() != $this->pseudo)
			{
				$ecrireFichier = true;
				$this->pseudo = $form->getPseudo();
				$nvxNomSession = $this->getNomSession() . $this->pseudo;
				$GLOBALS['connexion']->setNomSession($nvxNomSession);
				$_SESSION[$nvxNomSession] = true;
			}
			
			//mise à jours des champs
			foreach ($this->listeOption as $nomOption=>$option)
			{
				//is_array sert à éviter les hr qui sont utile uniquement pour la présentation
				if(is_array($option) && $form->get($nomOption) != $option[0])
				{
					if($this->estSpecifique($nomOption))
					{
						if($this->majSpecifique($nomOption, $form))
						{
							$ecrireFichier = true;
						}
					}
					else
					{
						$ecrireFichier = true;
						$this->listeOption[$nomOption][0] = $form->get($nomOption);
					}
				}
			}

			//si au moins une option a été changée, on réécrit dans le fichier de config
			if($ecrireFichier)
			{
				$this->ecrireFichierConf();
				$msg = $GLOBALS["langue"]->get("CONFIG_MODIFICATION_OK") . $msg;//"Modification enregistrés.<br>\n";
			}

			//si un nouveau logo a été envoyé
			$logo = new Logo($GLOBALS["langue"], CHEMIN_DOSSIER_LOGO);
			if(!$logo->verifUpload())
			{
				$msg .= $logo->getErreur();
			}
		}
		catch (FormulaireConfigurationException $e)
		{
			$msg .= $e->getMessage();
		}

		return $msg;
	}


	/**
	 * fonction qui écrit le fichier de configuration dans un fichier xml
	 */
	public function ecrireFichierConf()
	{
		if($this->fichierConf != "")
		{
			$htaccess  = "deny from all\n";
			$htaccess .= "<Files logo.png>\n";
			$htaccess .= "	allow from all\n";
			$htaccess .= "</Files>";
			verifHtaccess($this->fichierConf, $htaccess);
				
			//on rempli le fichier xml
			$handle = @fopen($this->fichierConf, "w+");
			
			$xml = new DOMDocument("1.0", "UTF-8");
			$xml->formatOutput = true;	//pour une indentation correcte (pas sur une seule ligne)
			
			$xmlConfig = $xml->createElement("config");
			$xml->appendChild($xmlConfig);
			
			$xmlPseudo = $xml->createElement("pseudo");
			$xmlPseudo_Data = $xml->createCDATASection($this->getPseudo());
			$xmlPseudo->appendChild($xmlPseudo_Data);
			$xmlConfig->appendChild($xmlPseudo);
			
			$xmlPass = $xml->createElement("pass");
			$xmlPass_Data = $xml->createCDATASection($this->getPassCrypte());
			$xmlPass->appendChild($xmlPass_Data);
			$xmlConfig->appendChild($xmlPass);
			
			//cas génériques
			foreach ($this->listeOption as $nomOption => $option)
			{
				if(is_array($option))
				{
					switch($option[1])
					{
						case Configuration::BOOLEAN :
							$xmlBool = $xml->createElement($nomOption, ($option[0] ? "true" : "false"));
							$xmlConfig->appendChild($xmlBool);
							break;
						case Configuration::STRING :
						case Configuration::LANGUE :
							$xmlStr = $xml->createElement($nomOption);
							$xmlStr_Data = $xml->createCDATASection($option[0]);
							$xmlStr->appendChild($xmlStr_Data);
							$xmlConfig->appendChild($xmlStr);
							break;
						case Configuration::ARRAY2 :
							$xmlArray = $xml->createElement($nomOption);
							foreach($option[0] as $nomRaccourci => $tabRaccourci)
							{
								$xmlRaccourci = $xml->createElement($nomRaccourci);
								$xmlStr_Data = $xml->createCDATASection($tabRaccourci[0]);
								$xmlRaccourci->appendChild($xmlStr_Data);
								$xmlArray->appendChild($xmlRaccourci);
							}
							
							$xmlConfig->appendChild($xmlArray);
							break;
						default :
							$xmlStr = $xml->createElement($nomOption, $option[0]);
							$xmlConfig->appendChild($xmlStr);
							break;
					}
				}
			}
			
			@fwrite($handle,$xml->saveXML());
			@fclose($handle);
		}
	}


	/**
	 * fonction qui va charger l'objet avec les données contenus dans le fichier xml
	 */
	public function charger()
	{
		if($this->fichierConf != "")
		{
			if(!file_exists($this->fichierConf))
			{
				$this->setPassCrypte(hashSB(""));
				$this->ecrireFichierConf();
			}
			else
			{
				$xmlLocal = new DOMDocument();

				//on va charger la config. Pour chaque propriété, on va vérifié que le noeud xml est bien présent avant de le lire et d'affecter la valeur
				if($xmlLocal->load($this->fichierConf))
				{
					if(count($xmlLocal->getElementsByTagName('pseudo'))>0 && isset($xmlLocal->getElementsByTagName('pseudo')->item(0)->nodeValue))
					{
						$this->pseudo = $xmlLocal->getElementsByTagName('pseudo')->item(0)->nodeValue;
					}
					
					if(count($xmlLocal->getElementsByTagName('pass'))>0 && isset($xmlLocal->getElementsByTagName('pass')->item(0)->nodeValue))
					{
						$this->passCrypte = $xmlLocal->getElementsByTagName('pass')->item(0)->nodeValue;
					}
					
					//cas génériques
					foreach ($this->listeOption as $nomOption => $option)
					{
						if(is_array($option))
						{
							if(count($xmlLocal->getElementsByTagName($nomOption))>0 && isset($xmlLocal->getElementsByTagName($nomOption)->item(0)->nodeValue))
							{
								switch($option[1])
								{
									case Configuration::ARRAY2 :
										$listeRacc = array();
										$xmlListeRacc =  $xmlLocal->getElementsByTagName($nomOption)->item(0)->childNodes;
										foreach($xmlListeRacc as $node)
										{
											if($node->nodeName != "#text")
											{
												$this->listeOption[$nomOption][0][$node->nodeName][0] = $node->nodeValue;
											}
										}
										break;
									case Configuration::BOOLEAN :
										$this->listeOption[$nomOption][0] = ($xmlLocal->getElementsByTagName($nomOption)->item(0)->nodeValue == "true" ? true : false);
										break;
									default :
										$this->listeOption[$nomOption][0] = $xmlLocal->getElementsByTagName($nomOption)->item(0)->nodeValue;
										break;
								}
							}
						}
					}
				}
			}
		}
	}
}



class FormulaireConfiguration
{
	protected $pseudo = "";
	protected $passAncien = "";
	protected $pass = "";
	protected $pass2 = "";
	
	protected $listeOption;

	public function get($nom)
	{
		return $this->listeOption[$nom][0];
	}

	public function getPseudo(){return $this->pseudo;}
	public function setPseudo($pseudo){$this->pseudo = $pseudo;}

	public function getPassAncien(){return $this->passAncien;}
	public function setPassAncien($passAncien){$this->passAncien = $passAncien;}

	public function getPass(){return $this->pass;}
	public function setPass($pass){$this->pass = $pass;}

	public function getPass2(){return $this->pass2;}
	public function setPass2($pass2){$this->pass2 = $pass2;}

	
	/**
	 * définie si un champs ne se comporte pas comme son type l'aurait voulu
	 * @param $nomOption	String	: nom du chmaps à tester
	 * @return boolean
	 */
	public function estSpecifique($nomOption)
	{
		return false;
	}
	
	
	/**
	 * traitement à effectuer si c'est un champs spécifique
	 * @param $nomOption	String	: nom du champs
	 * @return boolean
	 */
	public function traitementSpecifique($nomOption)
	{
		return false;
	}
	
	
	public function __construct($liste)
	{
		$this->listeOption = array();
		
		//initialisation des champs
		foreach ($liste as $nom => $option)
		{
			$this->listeOption[$nom] = $option;
			switch($option[1])
			{
				case Configuration::BOOLEAN :
					$this->listeOption[$nom][0] = false;
					break;
				case Configuration::FLOAT :
					$this->listeOption[$nom][0] = 0;
					break;
				case Configuration::STRING :
				case Configuration::LANGUE :
					$this->listeOption[$nom][0] = "";
					break;
			}
		}
		$msg = "";

		if(!empty($_POST['pseudo']))
		{
			$this->pseudo = $_POST['pseudo'];
		}

		if(!empty($_POST['passAncien']))
		{
			$this->passAncien = $_POST['passAncien'];
		}

		if(!empty($_POST['pass']))
		{
			$this->pass = $_POST['pass'];
		}

		if(!empty($_POST['pass2']))
		{
			$this->pass2 = $_POST['pass2'];
		}

		if(($this->pass != "" || $this->pass2 != "") && ($this->pass != $this->pass2))
		{
			$msg .= $GLOBALS["langue"]->get("CONFIG_FORM_ERROR_PASS") . "<br>";
		}
		
		//cas générique
		foreach($this->listeOption as $nomOption => $option)
		{
			if($this->estSpecifique($nomOption))
			{
				$this->traitementSpecifique($nomOption);
			}
			else
			{
				switch($option[1])
				{
					case Configuration::BOOLEAN :
						if(!empty($_POST[$nomOption]))
						{
							$this->listeOption[$nomOption][0] = true;
						}
						break;
					case Configuration::STRING :
						if(isset($_POST[$nomOption]))
						{
							$this->listeOption[$nomOption][0] = $_POST[$nomOption];
						}
						break;
					case Configuration::FLOAT :

						if(isset($_POST[$nomOption]) && is_numeric($_POST[$nomOption]))
						{
							$this->listeOption[$nomOption][0] = $_POST[$nomOption];
						}
						elseif(isset($this->listeOption[$nomOption][4]))
						{
							$msg .= $GLOBALS["langue"]->get($this->listeOption[$nomOption][4]) . "<br>";
						}
						break;
					case Configuration::LANGUE :
						if(!empty($_POST['slctLang']))
						{
							$this->listeOption[$nomOption][0] = $_POST['slctLang'];
						}
						break;
					case Configuration::ARRAY2 :
						$listeRacc = $this->listeOption[$nomOption][0];
						
						foreach($listeRacc as $nom => $arrayValeur)
						{
							if(!empty($_POST["racc" . $nom]))
							{
								$this->listeOption[$nomOption][0][$nom][0] = $_POST["racc" . $nom];
							}
						}
						break;
				}
			}
		}
		
		if($msg != "")
		{
			throw new FormulaireConfigurationException($msg);
		}
	}
}

class FormulaireConfigurationException extends Exception { }
?>
