<?php
/**
 * 
 *
 * Copyright 2012-2013 Superbaillot
 * This file is part of Sali.
 *
 *  Sali is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Sali is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Sali.  If not, see <http://www.gnu.org/licenses/>.
 */

class ConfigurationSali extends Configuration
{	
	protected $listeOption = array(
//									"nom"				=> array(init, type, label , description, message d'erreur) 
									"nbLienParPage"		=> array(20, Configuration::FLOAT, "CONFIG_FORM_NB_LIEN_LABEL", "CONFIG_FORM_NB_LIEN_DESC"),
									"recupereFavicon"	=> array(false, Configuration::BOOLEAN, "CONFIG_FORM_FAVICON_LABEL", "CONFIG_FORM_FAVICON_DESC"),
									"prive"			 	=> array(true, Configuration::BOOLEAN, "CONFIG_FORM_PRIVE_LABEL", "CONFIG_FORM_PRIVE_DESC", "CONFIG_FORM_ERROR_NB_ITEM_AFFICHE"),
									"hr1" 				=> true,
									"checkMaj"			=> array(true, Configuration::BOOLEAN, "CONFIG_FORM_CHECK_MAJ_LABEL", "CONFIG_FORM_CHECK_MAJ_DESC"),
									"langue"			=> array("fr", Configuration::LANGUE, "CONFIG_FORM_LANGUAGE_LABEL", "CONFIG_FORM_LANGUAGE_DESC"),
									"timezone"			=> array("Europe/Paris", Configuration::STRING, "CONFIG_FORM_TIMEZONE_CONTINENT_LABEL", "CONFIG_FORM_TIMEZONE_CONTINENT_DESC"),
								);
	

	//méthodes utilisées dans l'application. C'est plus facile à utiliser grâce à l'auto-complétion que l'accés via un tableau
	public function getNbLienParPage(){return $this->get("nbLienParPage");}
	public function getRecupereFavicon(){return $this->get("recupereFavicon");}
	public function getPrive(){return $this->get("prive");}
	public function getCheckMaj(){return $this->get("checkMaj");}
	public function getLangue(){return $this->get("langue");}
	public function getTimezone(){return $this->get("timezone");}
	
	
	public function getFormulaireConfiguration()
	{
		return new FormulaireConfigurationSali($this->listeOption);
	}
	
	public function estSpecifique($nom, $affichageSpecifique=false)
	{
		if($affichageSpecifique)
		{
			return in_array($nom, array("checkMaj", "timezone"));
		}
		else
		{
			return in_array($nom, array("langue", "timezone"));
		}
	}
	
	public function majSpecifique($nomOption, $form)
	{
		$return = false;
		switch($nomOption)
		{
			case "timezone" :
				$this->listeOption[$nomOption][0] = $form->get($nomOption);
				
				//je change le timezone
				date_default_timezone_set($this->get("timezone"));
				$return = true;
				break;
				
			case "langue" :
				if($form->get("langue") != $this->get("langue"))
				{
					$return = true;
					$this->listeOption["langue"][0] = $form->get("langue");
					$GLOBALS["langue"]->setNom($this->get("langue"));
				}
				break;
		}
		
		return $return;
	}
	
	public function affichageSpecifique($nomOption)
	{
		$out = "";
		switch($nomOption)
		{	
					case "checkMaj" :
						$out = "
						<div class=\"opt\">
						<input name=\"checkMaj[]\" id=\"checkMajtrue\" type=\"checkbox\" value=\"true\"" . ($this->getCheckMaj() ? " checked=\"checked\"" : "") . ">
						<label for=\"checkMajtrue\">" . $GLOBALS["langue"]->get("CONFIG_FORM_CHECK_MAJ_LABEL") . "</label>
					</div>
					<div class=\"desc\">
						" . $GLOBALS["langue"]->get("CONFIG_FORM_CHECK_MAJ_DESC", URL_CHECK_MAJ) . "
					</div>
						";
						break;
						
					case "timezone" :
						$out = "<div class=\"opt\">
						<label for=\"timezoneContinent\">" . $GLOBALS["langue"]->get("CONFIG_FORM_TIMEZONE_CONTINENT_LABEL") . "</label>
						<select id=\"timezoneContinent\" name=\"timezoneContinent\" onchange=\"majTimezoneVille(this.value)\">";
										
						list($continentActuel, $villeActelle) = explode("/", $this->get("timezone"), 2);
				
						$listeTz=timezone_identifiers_list();
						$lContinent = array();
						foreach ($listeTz as $tz)
						{
							$dataTz = explode("/", $tz, 2);
							if(count($dataTz)>1)
							{
								if(!isset($lContinent[$dataTz[0]]))
								{
									$lContinent[$dataTz[0]] = array();
								}
								$lContinent[$dataTz[0]][] = $dataTz[1];
							}
						}
				
						$js  = "var listeContinent = new Array();\n";
						$js .= "var villeActuelle = \"$villeActelle\";\n";
						
						foreach ($lContinent as $cont=>$lVille)
						{
							$js .= " listeContinent[\"$cont\"] = new Array(\"" . implode("\",\"", $lVille) . "\");\n";
							$out .= "
							<option value=\"$cont\"" . ($cont == $continentActuel ? " selected" : "") . ">
								$cont
							</option>"; 
						}
						$out .= "
						</select>
						<label for=\"timezoneVille\">; Ville</label>
						
						<select id=\"timezoneVille\" name=\"timezoneVille\">
						</select>
						
						<script type=\"text/javascript\">
						$js
						majTimezoneVille(\"$continentActuel\");
						</script>
					</div>
					<div class=\"desc\">
						" . $GLOBALS["langue"]->get("CONFIG_FORM_TIMEZONE_DESC") . "
					</div>";
						break;
		}
		
		return $out;
	}
}



class FormulaireConfigurationSali extends FormulaireConfiguration
{
	public function estSpecifique($nomOption)
	{
		return in_array($nomOption, array("timezone"));
	}
	
	public function traitementSpecifique($nomOption)
	{
		switch($nomOption)
		{
			case "timezone" :
				if(!empty($_POST['timezoneContinent']) && !empty($_POST['timezoneVille']))
				{
					$this->listeOption["timezone"][0] = $_POST['timezoneContinent'] . "/" . $_POST['timezoneVille'];
				}

				break;
		}
	}
	
}
?>
