<?php
/**
 * 
 *
 * Copyright 2012-2013 Superbaillot
 * This file is part of Sali.
 *
 *  Sali is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Sali is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Sali.  If not, see <http://www.gnu.org/licenses/>.
 */



class Lien
{
	public static $LARGEUR_REFERENCE = 200;
	public static $HAUTEUR_REFERENCE = 150;
	public static $CLE = "SALI";
	
	private $id = 0;
	private $url = "";
	private $titre = "";
	private $commentaire = "";
	private $date = "";
	private $etat = 0;
	private $favicon = "";
	private $listeTag = array();


	public function setId($id){$this->id = $id;}
	public function getId(){return $this->id;}

	public function setUrl($url){$this->url = $url;}
	public function getUrl(){return $this->url;}
	public function getUrlAffichee()
	{
		if($this->url == "")
		{
			return $this->getPermalien();
		}
		return $this->url;
	}
	
	public function getTitre(){return $this->titre;}
	public function setTitre($titre){$this->titre = $titre;}

	public function setCommentaire($commentaire){$this->commentaire = $commentaire;}
	public function getCommentaire(){return $this->commentaire;}
	
	public function getDate(){return $this->date;}
	public function setDate($date){$this->date = $date;}

	public function setEtat($etat){$this->etat = $etat;}
	public function getEtat(){return $this->etat;}

	public function setFavicon($favicon){$this->favicon = $favicon;}
	public function getFavicon(){return $this->favicon;}
	
	public function getListeTag(){return $this->listeTag;}
	public function setListeTag($listeTag){$this->listeTag = $listeTag;}
	
	public function getPermalien()
	{
		return getServeur() . "?id=" . $this->getId();
	}

	/**
	 * fonction qui ajoute une liste de tag (séparés par des ,) à la liste actuelle de tag
	 * @param $tag	String
	 */
	public function ajouteTag($tag)
	{
		$liste = explode(",", $tag);
		foreach($liste as $t)
		{
			$t = trim(mb_convert_case($t, MB_CASE_TITLE, "UTF-8"));
			if(!in_array($t, $this->listeTag))
			{
				if($t != "")
				{
					$this->listeTag[] = $t;
				}
			}
		}
	}
	
	
	/**
	 * méthode qui construit l'appérçu du lien. Pour l'instant, que les images sont pris en compte.
	 */	 	
	public function getAppercu()
	{
		$return = "";
		$urlImg = "";
		$htAccess="<Files *.jpg>
	allow from all
</Files>
<Files *.jpeg>
	allow from all
</Files>
<Files *.png>
	allow from all
</Files>
<Files *.gif>
	allow from all
</Files>";
		verifHtaccess(REPERTOIRE_VIGNETTE, $htAccess);
		
		$ext = strtolower(pathinfo($this->getUrl(), PATHINFO_EXTENSION));
		switch ($ext)
		{
			case "jpg":
			case "jpeg":
				$urlImg = REPERTOIRE_VIGNETTE . $this->getId() . ".$ext";
				if(!file_exists(REPERTOIRE_VIGNETTE . $this->getId() . ".$ext"))
				{
					$source = @imagecreatefromjpeg($this->getUrl());
					$im = $this->recupereImage($source, $ext);
					imagejpeg($im, $urlImg);
				}
			case "png":
				$urlImg = REPERTOIRE_VIGNETTE . $this->getId() . ".$ext";
				if(!file_exists(REPERTOIRE_VIGNETTE . $this->getId() . ".$ext"))
				{
					$source = @imagecreatefrompng($this->getUrl());
					$im = $this->recupereImage($source, $ext);
					imagepng($im, $urlImg);
				}
			case "gif":
				$urlImg = REPERTOIRE_VIGNETTE . $this->getId() . ".$ext";
				if(!file_exists(REPERTOIRE_VIGNETTE . $this->getId() . ".$ext"))
				{
					$source = @imagecreatefromgif($this->getUrl());
					$im = $this->recupereImage($source, $ext);
					imagegif($im, $urlImg);
				}
		}
		if($urlImg != "")
		{
			$return = "
			<a href='" . $this->getUrl() .  "' target='_blank' title='" . $this->getUrl() .  "'>
				<img src=\"" . getServeur(false) . "$urlImg\" title=\"" . htmlspecialchars($this->getTitre()) . "\" alt=\"" . htmlspecialchars($this->getTitre()) . "\">
			</a>\n";
		}
		return $return;
	}


	/**
	 * fonction qui fait une copie de l'apperçu d'une image
	 * @param $source
	 * @param $extension
	 */
	public function recupereImage($source, $extension)
	{
		//récupération des dimentions de l'image originale
		$largeurSource = imagesx($source);
		$hauteurSource = imagesy($source);

		//si une des 2 dimensions est trop grande, on va réduire la taille tout en gardant les proportions
		if($largeurSource > Lien::$LARGEUR_REFERENCE || $hauteurSource > Lien::$HAUTEUR_REFERENCE)
		{
			$ratioReference = Lien::$LARGEUR_REFERENCE / Lien::$HAUTEUR_REFERENCE;
			$ratio = $largeurSource / $hauteurSource;
			
			//si l'image est dans le sens de la hauteur, il faut redimentionner l'image avec la hauteur en reference
			if($ratio < $ratioReference)
			{
				//calcul des dimensions de l'image
				$hauteurDestination = Lien::$HAUTEUR_REFERENCE;

				// Contraint le rééchantillonage à une largeur fixe
				// Maintient le ratio de l'image
				$largeurDestination = round(($hauteurDestination / $hauteurSource) * $largeurSource);
				
				//au delà d'une certaine proportion, on coupe la fin de l'image car l'image est tellement fine que l'on y vera plus rien
				if($largeurDestination < Lien::$LARGEUR_REFERENCE / 2)
				{
					$hauteurDestination = Lien::$HAUTEUR_REFERENCE;
					$largeurDestination = Lien::$LARGEUR_REFERENCE / 2;
					$hauteurSource = round(($largeurSource / $largeurDestination) * $hauteurDestination);
				}
			}
			else
			{
				//calcul des dimensions de l'image
				$largeurDestination = Lien::$LARGEUR_REFERENCE;

				// Contraint le rééchantillonage à une largeur fixe
				// Maintient le ratio de l'image
				$hauteurDestination = round(($largeurDestination / $largeurSource) * $hauteurSource);
				
				//au delà d'une certaine proportion, on coupe la fin de l'image car l'image est tellement fine que l'on y vera plus rien
				if($hauteurDestination < Lien::$HAUTEUR_REFERENCE / 1.5)
				{
					$hauteurDestination = Lien::$HAUTEUR_REFERENCE / 1.5;
					$largeurDestination = Lien::$LARGEUR_REFERENCE;
					$largeurSource = round(($hauteurSource / $hauteurDestination) * $largeurDestination);
				}
			}
		}
		else
		{
			$hauteurDestination = $hauteurSource;
			$largeurDestination = $largeurSource;
		}

		//création de l'image redimensionnée
		$im =  imagecreatetruecolor ($largeurDestination, $hauteurDestination);

		//on met le background en transparent
		imagesavealpha($im, true);
		$background = imagecolorallocatealpha ($im ,0 ,127 ,0 ,127);
		imagefill( $im, 0, 0, $background );

		//copie de l'image redimensionnée dans le nouveau fichier
		ImageCopyResampled($im, $source, 0, 0, 0, 0, $largeurDestination, $hauteurDestination, $largeurSource, $hauteurSource);
		
		return $im;
	}


	/**
	 * méthode qui récupère le favicon du lien
	 */
	public function recupreFavicon()
	{
		if($this->url == "")
		{
			$this->favicon = CHEMIN_FAVICON_SALI;
		}
		else
		{
			$favIcon = new Favicon($this->url);
			$urlFav = $favIcon->getFavicon();

			//si on a un favicon, on le copie
			if($urlFav != "")
			{
				//on ne veut que l'extension du fichier
				$nomFichier = "favicon" . $this->getId() . "." . pathinfo($urlFav, PATHINFO_EXTENSION);
				$newfile = REPERTOIRE_FAVICON . $nomFichier;
				verifChemin($newfile);

				//s'il n'existe pas, on crée le .htaccess qui autorise l'acces aux favicon (il y a une autre .htaccess plus bat dans la hierarchie qui interdit l'acces à tous les fichiers par défaut)
				verifHtaccess(REPERTOIRE_FAVICON, "allow from all\n");

				copy($urlFav, $newfile);
				$this->favicon = $nomFichier;
			}
		}
	}
	
	
	/**
	 * méthode qui construit le xml du lien pour l'enregistrer
	 * @return BaliseXml
	 */
	public function toXML()
	{	
		$xml  = "	<lien>\n";
		$xml .= "		<id>" . crypte($this->getId(), Lien::$CLE) . "</id>\n";
		$xml .= "		<url><![CDATA[" . crypte($this->getUrl(), Lien::$CLE) . "]]></url>\n";
		$xml .= "		<titre><![CDATA[" . crypte($this->getTitre(), Lien::$CLE) . "]]></titre>\n";
		$xml .= "		<commentaire><![CDATA[" . crypte($this->getCommentaire(), Lien::$CLE) . "]]></commentaire>\n";
		$xml .= "		<favicon><![CDATA[" . crypte($this->getFavicon(), Lien::$CLE) . "]]></favicon>\n";
		$xml .= "		<etat>" . crypte($this->getEtat(), Lien::$CLE) . "</etat>\n";
		$xml .= "		<date>" . crypte($this->getDate(), Lien::$CLE) . "</date>\n";
		if(count($this->getListeTag()) > 0 )
		{
			foreach($this->getListeTag() as $tag)
			{
				$xml .= "		<tag>" . crypte($tag, Lien::$CLE) . "</tag>\n";
			}
		}
		$xml .= "	</lien>\n";

		return $xml;
	}
	
	
	/**
	 * méthode qui charge un lien avec le noeud xml
	 * @param $xmlLien
	 * @param $listeTag
	 * @param $listeCat		Array()	: utilisé uniquement pour les anciennes sauveardes pour transformer les catégories en tags
	 * @return String : message
	 */
	public function charge($xmlLien, &$listeTag, $listeCat)
	{
		$etat = $xmlLien->getElementsByTagName('etat')->item(0)->nodeValue;
		
		//ancienne sauvegarde pas cryptées ( < 3.1 )
		if(is_numeric($etat))
		{
			$this->setEtat($etat);

			//on charge le lien que s'il est public ou que l'on est connecté
			if($this->getEtat() == ETAT_VISIBLE_TOUS || $GLOBALS["connexion"]->estConnecte())
			{
				$this->setId($xmlLien->getElementsByTagName('id')->item(0)->nodeValue);
				$this->setUrl($xmlLien->getElementsByTagName('url')->item(0)->nodeValue);
				$this->setCommentaire($xmlLien->getElementsByTagName('commentaire')->item(0)->nodeValue);
				if($xmlLien->getElementsByTagName('titre')->length>0)
				{
					$this->setTitre($xmlLien->getElementsByTagName('titre')->item(0)->nodeValue);
				}
				else//pour les anciennes sauvegardes
				{
					$this->setTitre($xmlLien->getElementsByTagName('commentaire')->item(0)->nodeValue);
				}
				if($xmlLien->getElementsByTagName('favicon')->length>0)
				{
					$this->setFavicon($xmlLien->getElementsByTagName('favicon')->item(0)->nodeValue);
				}
				if($xmlLien->getElementsByTagName('date')->length>0)
				{
					$this->setDate($xmlLien->getElementsByTagName('date')->item(0)->nodeValue);
				}
				else
				{
					$this->setDate(date("Y-m-d H:i"));
				}
				if($xmlLien->getElementsByTagName('tag')->length>0)
				{
					foreach($xmlLien->getElementsByTagName('tag') as $xmlTag)
					{
						$tag = $xmlTag->nodeValue;
						$this->ajouteTag($tag);

						//je m'assure qu'il y ait bien une majuscule à chaque tag (c'est surtout utile pour les anciennes sauvegardes, car c'est aussi fait à l'ajout d'un nouveau lien)
						$tag = mb_convert_case($tag, MB_CASE_TITLE, "UTF-8");

						if(!isset($listeTag["$tag"]))
						{
							$listeTag["$tag"] = 1;
						}
						else
						{
							$listeTag["$tag"]++;
						}
					}
				}

				//pour les anciennes sauvegardes ( < 3.0)
				if($xmlLien->getElementsByTagName('categorieId')->length>0)
				{
					$tag = $listeCat["cat" . $xmlLien->getElementsByTagName('categorieId')->item(0)->nodeValue];
					$this->ajouteTag($tag);
						
					if(!isset($listeTag["$tag"]))
					{
						$listeTag["$tag"] = 1;
					}
					else
					{
						$listeTag["$tag"]++;
					}
				}
			}
		}
		
		//sauvegarde cryptée
		else
		{
			$this->setEtat(decrypte($etat, Lien::$CLE));

			//on charge le lien que s'il est public ou que l'on est connecté
			if($this->getEtat() == ETAT_VISIBLE_TOUS || $GLOBALS["connexion"]->estConnecte())
			{
				$this->setId(decrypte($xmlLien->getElementsByTagName('id')->item(0)->nodeValue, Lien::$CLE));
				$this->setUrl(decrypte($xmlLien->getElementsByTagName('url')->item(0)->nodeValue, Lien::$CLE));
				$this->setCommentaire(decrypte($xmlLien->getElementsByTagName('commentaire')->item(0)->nodeValue, Lien::$CLE));
				if($xmlLien->getElementsByTagName('titre')->length>0)
				{
					$this->setTitre(decrypte($xmlLien->getElementsByTagName('titre')->item(0)->nodeValue, Lien::$CLE));
				}
				else//pour les anciennes sauvegardes
				{
					$this->setTitre(decrypte($xmlLien->getElementsByTagName('commentaire')->item(0)->nodeValue, Lien::$CLE));
				}
				if($xmlLien->getElementsByTagName('favicon')->length>0)
				{
					$this->setFavicon(decrypte($xmlLien->getElementsByTagName('favicon')->item(0)->nodeValue, Lien::$CLE));
				}
				if($xmlLien->getElementsByTagName('date')->length>0)
				{
					$this->setDate(decrypte($xmlLien->getElementsByTagName('date')->item(0)->nodeValue, Lien::$CLE));
				}
				else
				{
					$this->setDate(date("Y-m-d H:i"));
				}
				if($xmlLien->getElementsByTagName('tag')->length>0)
				{
					foreach($xmlLien->getElementsByTagName('tag') as $xmlTag)
					{
						$tag = decrypte($xmlTag->nodeValue, Lien::$CLE);
						$this->ajouteTag($tag);

						//je m'assure qu'il y ait bien une majuscule à chaque tag (c'est surtout utile pour les anciennes sauvegardes, car c'est aussi fait à l'ajout d'un nouveau lien)
						$tag = mb_convert_case($tag, MB_CASE_TITLE, "UTF-8");

						if(!isset($listeTag["$tag"]))
						{
							$listeTag["$tag"] = 1;
						}
						else
						{
							$listeTag["$tag"]++;
						}
					}
				}

				//pour les anciennes sauvegardes
				if($xmlLien->getElementsByTagName('categorieId')->length>0)
				{
					$tag = $listeCat["cat" . $xmlLien->getElementsByTagName('categorieId')->item(0)->nodeValue];
					$this->ajouteTag($tag);
						
					if(!isset($listeTag["$tag"]))
					{
						$listeTag["$tag"] = 1;
					}
					else
					{
						$listeTag["$tag"]++;
					}
				}
			}
		}
	}
}


/**
 * fonction qui va chercher un liens dans la liste en fonction de son id.
 * @param $id	int	: id du lien recherché
 * @return Lien
 */
function getLienById_sbd($id)
{
	global $listeLien;

	$lien = new Lien();

	foreach($listeLien as &$li)
	{
		if($id == $li->getId())
		{
			return $li;
		}
	}

	return $lien;
}


/**
 * fonction qui va chercher un liens dans la liste en fonction de son $url.
 * @param $url	String	: $url du lien recherché
 * @return Lien
 */
function getLienByUrl_sbd($url)
{
	global $listeLien;

	$lien = new Lien();

	foreach($listeLien as &$li)
	{
		if($url == $li->getUrl())
		{
			return $li;
		}
	}

	return $lien;
}


/**
 * fonction qui renvoie la liste des liens d'une catégorie
 * @param $filtre	RechercheSali	: filtre des liens
 * @return array(Lien)
 */
function getListeLiens_sbd(RechercheSali $filtre)
{
	global $listeLien;
	$liste = array();
	$nbTagRecherche = count($filtre->getListeTag());
	$nbMotRecherche = count($filtre->getListeMot());
	
	if(count($listeLien)>0)
	{
		foreach($listeLien as $lien)
		{
			$cptTagTrouve = 0;
			$cptMotTrouve = 0;
			if($nbMotRecherche > 0) 
			{
				foreach($filtre->getListeMot() as $mot)
				{
					//on recherche le mot dans le titre et le commentaire, non cas-sensitive
					if( ($lien->getTitre() != "" && stripos($lien->getTitre(), $mot) !== false)
						|| ($lien->getCommentaire() != "" && stripos($lien->getCommentaire(), $mot) !== false))
					{
						if($nbTagRecherche > 0)
						{
							foreach($filtre->getListeTag() as $tag)
							{
								if(estDansTag($lien->getListeTag(), $tag))
								{
									$cptTagTrouve++;
									//si tous les tags ont été trouvés, le mot à donc été trouvé
									if($nbTagRecherche == $cptTagTrouve)
									{
										$cptMotTrouve++;
									}
								}
							}
						}
						else
						{
							$cptMotTrouve++;
						}
						
						//si on a trouver le tous les mots et tous les tags, c'est bon
						if($nbMotRecherche == $cptMotTrouve && $nbTagRecherche == $cptTagTrouve)
						{
							$liste[] = $lien;
						}
					}
				}
			}
			elseif($nbTagRecherche > 0)
			{
				$cptTagTrouve = 0;
				foreach($filtre->getListeTag() as $tag)
				{
					if(estDansTag($lien->getListeTag(), $tag))
					{
						$cptTagTrouve++;

						//si on a trouve tous les tags, c'est bon
						if($cptTagTrouve == $nbTagRecherche)
						{
							$liste[] = $lien;
						}
					}
				}
			}
			else
			{
				$liste[] = $lien;
			}
		}
	}

	return $liste;
}



function estDansTag($arrayTag, $mot, $caseSensitive=false)
{
	if(count($arrayTag) > 0)
	{
		foreach($arrayTag as $tag)
		{
			if(($caseSensitive && strcasecmp($tag, $mot) == 0)
				|| (!$caseSensitive && strcmp($tag, $mot) == 0))
			{
				return true;
			}
		}
	}
	return false;
}
?>
