<?php
/**
 *
 *
 * Copyright 2012-2013 Superbaillot
 * This file is part of Sali.
 *
 *  Sali is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Sali is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Sali.  If not, see <http://www.gnu.org/licenses/>.
 */


class Sali
{
	private $liste = array();
	private $listeIdUtilise = array();
	private $listeTag = array();
	private $fichier = "";

	public function getListe(){return $this->liste;}
	public function setListe($liste){$this->liste = $liste;}

	public function getListeIdUtilise(){return $this->listeIdUtilise;}
	public function setListeIdUtilise($listeIdUtilise){$this->listeIdUtilise = $listeIdUtilise;}

	public function getListeTag(){return $this->listeTag;}
	public function setListeTag($listeTag){$this->listeTag = $listeTag;}

	public function getFichier(){return $this->fichier;}
	public function setFichier($fichier){$this->fichier = $fichier;}


	public function __construct($fichier)
	{
		$this->setFichier($fichier);
	}

	public function charger()
	{
		$msg = "";
		$listeLien = array();
		if(file_exists($this->getFichier()))
		{
			$xmlLocal = new DOMDocument();

			//si c'est un xml valide
			if($xmlLocal->load($this->getFichier()))
			{
				//pour les anciennes sauvegardes, on charge les catégories pour les mettre en tag

				$listeCat = array();
				//on charge toutes les catégories
				foreach($xmlLocal->getElementsByTagName('categorie') as $xmlCat)
				{
					$listeCat["cat" . $xmlCat->getElementsByTagName('id')->item(0)->nodeValue] = $xmlCat->getElementsByTagName('nom')->item(0)->nodeValue;
				}
					
				//on charge tous les liens
				foreach($xmlLocal->getElementsByTagName('lien') as $xmlLien)
				{
					$lien = new Lien();
					$lien->charge($xmlLien, $this->listeTag, $listeCat);
						
					if($lien->getId() > 0)
					{
						$listeLien[] = $lien;
						$this->listeIdUtilise["id" . $lien->getId()] = $lien->getId();
					}
				}
				usort($listeLien, "triParDate");
				$this->setListe($listeLien);
			}
			else
			{
				$msg = $GLOBALS["langue"]->get("MSG_NON_VALIDE");
			}
		}
		else
		{
			$msg = $GLOBALS["langue"]->get("MSG_NON_VALIDE");
		}
		return $msg;
	}


	/**
	 * méthode qui supprime le lien de la liste, mais supprimer aussi le favicon et l'apperçu s'ils existent
	 * @param $id
	 */
	public function ajouterLien(Lien $lien, $chargerFavicon=true, $sauvegarder=true)
	{
		$lien->setId($this->getIdSuivant());

		//lors d'un import, la date est déjà saisie et il faut la garder
		if($lien->getDate() == "")
		{
			$lien->setDate(date("Y-m-d H:i"));
		}

		//on récupère le favicon si on l'a configurer ainsi
		if($chargerFavicon && $GLOBALS["config"]->getRecupereFavicon())
		{
			$lien->recupreFavicon();
		}

		$this->liste[] = $lien;
		usort($this->liste, "triParDate");

		if($sauvegarder)
		{
			$this->enregistrer();
		}
		return $lien;
	}


	/**
	 * méthode qui supprime le lien de la liste, mais supprimer aussi le favicon et l'apperçu s'ils existent
	 * @param $id
	 */
	public function supprimerLien($id)
	{
		$lien = $this->getLienById($id);

		if($lien->getId() != 0)
		{
			//si on a un favicon, on le supprime
			if($lien->getFavicon() != "" && file_exists(REPERTOIRE_FAVICON . $lien->getFavicon()))
			{
				unlink(REPERTOIRE_FAVICON . $lien->getFavicon());
			}

			//si on a un apperçu, on le supprime
			$nom = REPERTOIRE_VIGNETTE . $lien->getId() . "." . pathinfo($lien->getUrl(), PATHINFO_EXTENSION);
			if(file_exists($nom))
			{
				unlink($nom);
			}

			//on supprime le lien de la liste
			unset($this->liste[array_search($lien,$this->liste)]);

			//on sauvegarde le changement
			$this->enregistrer();
				
			return true;
		}

		return false;
	}



	/**
	 * fonction qui retourne le premier id non utilisé
	 * Lorsque l'on utilise cette fonction, l'id est automatiquement marqué comme utilisé.
	 * @return int
	 */
	public function getIdSuivant()
	{
		$id = 1; //id suivant potentiel. Si je le trouve dans le tableau, j'essaye le suivant
		$trouver = false;

		//j'essaie tous les id à partir de 1 et je teste s'il est dans la liste. S'il n'y est pas, ça sera la valeur de l'id suivant
		while(!$trouver && $id <= count($this->listeIdUtilise))
		{
			if(!isset($this->listeIdUtilise["id$id"]))
			{
				$trouver = true;
			}
			else
			{
				$id++;
			}
		}
		$this->listeIdUtilise["id$id"] = $id;

		return $id;
	}


	/**
	 * fonction qui sauvegarde les listes de liens et de catégories dans un fichier
	 * @return rien
	 */
	public function enregistrer()
	{
		//on rempli le fichier xml
		$handle = @fopen($this->getFichier(), "w+");
		$xmlConfig  = "<?xml version='1.0' encoding='UTF-8'?>\n";
		$xmlConfig .= "<listeLien>\n";

		foreach($this->getListe() as $lien)
		{
			if($lien->getId()>0)
			{
				$xmlConfig .= $lien->toXML();
			}
		}
		$xmlConfig .= "</listeLien>\n";
		@fwrite($handle,$xmlConfig);
		@fclose($handle);
	}


	/**
	 * fonction qui génère le fichier d'export au format NETSCAPE-Bookmark-file
	 */
	public function export()
	{
		$final_xml  = "<!DOCTYPE NETSCAPE-Bookmark-file-1>\n";
		$final_xml .= "<META HTTP-EQUIV='Content-Type' CONTENT='text/html; charset=UTF-8'>\n";
		$final_xml .= "<!--This is an automatically generated file.\n";
		$final_xml .= "It will be read and overwritten.\n";
		$final_xml .= "Do Not Edit! -->\n";
		$final_xml .= "<TITLE>Sali</TITLE>\n";
		$final_xml .= "<H1>Sali</H1>\n";

		foreach ($this->getListe() as $lien)
		{
			$url = $lien->getUrl();
			if($url == "")
			{
				$url = getServeur() . "?id=" . $lien->getId();
			}
			$final_xml .= "<DT><A HREF=\"$url\" ADD_DATE=\"" . strtotime($lien->getDate()) . "\" PRIVATE=\"" . ($lien->getEtat() == ETAT_VISIBLE_ADMIN ? "1" : "0") . "\" TAGS=\"" . implode(",", $lien->getListeTag()) . "\">" . $lien->getTitre() . "</A>\n";
			$final_xml .= "<DD>" . $lien->getCommentaire();
		}

		header("Cache-Control: public");
		header("Content-Description: File Transfer");
		header("Content-Disposition: attachment; filename=Sali.html");
		header("Content-Type: text/html; charset='utf8'");
		header("Content-Transfer-Encoding: binary");

		echo $final_xml;

		exit; // nécessaire pour être certain de ne pas envoyer de fichier corrompu
	}


	/**
	 * fonction qui importe un fichier au format NETSCAPE-Bookmark-file
	 * @return String : message d'état
	 */
	public function import()
	{
		$filename=$_FILES['import']['name'];
		$filesize=$_FILES['import']['size'];
		$data=file_get_contents($_FILES['import']['tmp_name']);
		$forcePrive = !empty($_POST['prive']);
		$cpt=0;
		set_time_limit(60);

		//NETSCAPE-Bookmark-file
		if (startsWith($data,'<!DOCTYPE NETSCAPE-Bookmark-file-1>'))
		{
			foreach(explode('<DT>',$data) as $html)
			{
				$lien = new Lien();
					
				$dd = explode('<DD>',$html);
				if (startswith($dd[0],'<A '))
				{
					$commentaire = (isset($dd[1]) ? html_entity_decode(trim($dd[1]),ENT_QUOTES,"UTF-8") : '');
						
					$lien->setCommentaire($commentaire);
						
					preg_match("!<A .*?>(.*?)</A>!i",$dd[0],$matches);
					$title = (isset($matches[1]) ? trim($matches[1]) : "");
					$lien->setTitre(html_entity_decode($title,ENT_QUOTES,"UTF-8"));

					preg_match_all("! ([A-Z_]+)=[\"'](.*?)[\"']!i",$html,$matches,PREG_SET_ORDER);
					$date=0;
					foreach($matches as $m)
					{
						$attr=$m[1];
						$value=$m[2];
						if ($attr=="HREF")
						{
							$lien->setUrl(html_entity_decode($value,ENT_QUOTES,"UTF-8"));
						}
						elseif ($attr=="ADD_DATE")
						{
							$date=intval($value);
						}
						elseif ($attr=="PRIVATE")
						{
							$lien->setEtat(($value=="1" || $forcePrive) ?ETAT_VISIBLE_ADMIN:ETAT_VISIBLE_TOUS);
						}
						elseif ($attr=="TAGS")
						{
							$lien->ajouteTag(html_entity_decode($value,ENT_QUOTES,"UTF-8"));
						}
					}

					if (empty($date))
					{
						$date=time();
					}

					$lien->setDate(date("Y-m-d H:i", $date));

					$lienTemp = $this->getLienByUrl($lien->getUrl());

					//si le lien n'existe pas, on l'ajoute
					if($lienTemp->getId() == 0)
					{
						set_time_limit($cpt * 0.2);
						$this->ajouterLien($lien, false, false);	//trop long de charger tous les favicons et pour sauvegarder à chaque fois
						$cpt++;

						//tous les 50 liens, je sauvegarde pour le cas où j'ai un timeout, on pourra réimporter la suite, sans tout recommencer à 0.
						if($cpt % 50 == 0)
						{
							$this->enregistrer();
						}
					}
				}
			}
			$this->enregistrer();
			return $GLOBALS["langue"]->get("MSG_IMPORT_OK", $filename, $cpt);
		}
		else
		{
			return $GLOBALS["langue"]->get("MSG_IMPORT_KO", $filename);
		}
	}


	/**
	 * fonction qui va chercher un liens dans la liste en fonction de son id.
	 * @param $id	int	: id du lien recherché
	 * @return Lien
	 */
	public function getLienById($id)
	{
		$lien = new Lien();

		foreach($this->liste as &$li)
		{
			if($id == $li->getId())
			{
				return $li;
			}
		}

		return $lien;
	}


	/**
	 * fonction qui va chercher un liens dans la liste en fonction de son $url.
	 * @param $url	String	: $url du lien recherché
	 * @return Lien
	 */
	public function getLienByUrl($url)
	{
		$lien = new Lien();

		foreach($this->liste as &$li)
		{
			if($url == $li->getUrl())
			{
				return $li;
			}
		}

		return $lien;
	}


	/**
	 * fonction qui renvoie la liste des liens filtrés
	 * @param $filtre	RechercheSali	: filtre des liens
	 * @return array( Lien )
	 */
	public function getListeLiens(RechercheSali $filtre)
	{
		$liste = array();
		$nbTagRecherche = count($filtre->getListeTag());
		$nbMotRecherche = count($filtre->getListeMot());
		$nbExt = count($filtre->getTypeLien());

		if(count($this->getListe())>0)
		{
			foreach($this->getListe() as $lien)
			{
				$cptTagTrouve = 0;
				$cptMotTrouve = 0;
				if($nbMotRecherche > 0)
				{
					foreach($filtre->getListeMot() as $mot)
					{
						//on recherche le mot dans le titre et le commentaire et l'url, non cas-sensitive
						if( ($lien->getTitre() != "" && stripos($lien->getTitre(), $mot) !== false)
						|| ($lien->getCommentaire() != "" && stripos($lien->getCommentaire(), $mot) !== false)
						|| ($lien->getUrl() != "" && stripos($lien->getUrl(), $mot) !== false))
						{
							if($nbTagRecherche > 0)
							{
								foreach($filtre->getListeTag() as $tag)
								{
									if(estDansTag($lien->getListeTag(), $tag))
									{
										$cptTagTrouve++;
										//si tous les tags ont été trouvés, le mot à donc été trouvé
										if($nbTagRecherche == $cptTagTrouve)
										{
											$cptMotTrouve++;
										}
									}
								}
							}
							else
							{
								$cptMotTrouve++;
							}
								
							//si on a trouver tous les mots et tous les tags et s'il y a un fliltre sur l'extention et qu'il est respecté, c'est bon
							if($nbMotRecherche == $cptMotTrouve && $nbTagRecherche == $cptTagTrouve && ($nbExt == 0 || in_array(pathinfo($lien->getUrl(), PATHINFO_EXTENSION), $filtre->getTypeLien())))
							{
								$liste[] = $lien;
							}
						}
					}
				}
				elseif($nbTagRecherche > 0)
				{
					$cptTagTrouve = 0;
					foreach($filtre->getListeTag() as $tag)
					{
						if(estDansTag($lien->getListeTag(), $tag))
						{
							$cptTagTrouve++;

							//si on a trouve tous les tags et s'il y a un fliltre sur l'extention et qu'il est respecté, c'est bon
							if($cptTagTrouve == $nbTagRecherche && ($nbExt == 0 || in_array(pathinfo($lien->getUrl(), PATHINFO_EXTENSION), $filtre->getTypeLien())))
							{
								$liste[] = $lien;
							}
						}
					}
				}
				elseif($nbExt == 0 || in_array(pathinfo($lien->getUrl(), PATHINFO_EXTENSION), $filtre->getTypeLien()))
				{
					$liste[] = $lien;
				}
			}
		}

		return $liste;
	}
}
?>
