<?php
/**
 * 
 *
 * Copyright 2012-2013 Superbaillot
 * This file is part of Sali.
 *
 *  Sali is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Sali is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Sali.  If not, see <http://www.gnu.org/licenses/>.
 */

define("CHEMIN_IMPORT_OM_CONFIGURATION", CHEMIN_IMPORT_OM_SALI . "configuration.class.php");
define("CHEMIN_IMPORT_OM_CONNEXION", CHEMIN_IMPORT_OM_SALI . "connexion.class.php");
define("CHEMIN_IMPORT_OM_LOGO", CHEMIN_IMPORT_OM_SALI . "logo.class.php");
define("CHEMIN_IMPORT_OM_FAVICON", CHEMIN_IMPORT_OM_SALI . "favicon.class.php");
define("CHEMIN_IMPORT_OM_LANGUE", CHEMIN_IMPORT_OM_SALI . "langue.class.php");
define("CHEMIN_SERVEUR","http://superbaillot.net");

define("CHEMIN_IMPORT_IMAGE","DOC/Images/");
define("CHEMIN_FAVICON_SALI", CHEMIN_IMPORT_IMAGE ."saliFavicon.png");

define("CHEMIN_CSS_DEFAULT", CHEMIN_CSS . "default.css");
define("CHEMIN_IMPORT_AWESOME_FONT", "DOC/font-awesome/css/font-awesome.min.css");

//require_once(CHEMIN_IMPORT_RUBIH);
//require_once(CHEMIN_IMPORT_RUBIH_SPECIFIC);

/**
 * Fonction qui vérifie si les répertoires du chemin spécifié existent.
 * S'ils n'existent pas, on les crée
 * @param $chemin	String	: chemin à vérifier. il peut s'agir d'un répertoire ou d'un fichier
 * @param $creation	boolean	: s'il s'agit d'un fichier, cet argument va définir s'il faut créer le fichier vide
 * @return boolean
 */
function verifChemin($chemin, $creation=false)
{
	$noProbleme = true;
	$listeRep = explode("/", $chemin);
	
	$nb = count($listeRep);
	
	$cheminVerif="";
	
	//on vérifie le chemin de tous les éléments sauf du dernier car le dernier peut être soit un répertoire, soit un fichier
	for($i=0; $i<$nb-1; $i++)
	{
		//on construit le chemin à vérifier.
		//On ajoute / pour séparer les répertoires uniquement si on est pas à la racine du répertoire à vérifier
		$cheminVerif = $cheminVerif . ($cheminVerif==""?"":"/") . $listeRep[$i];
		
		//si le répertoire à vérifier n'existe pas, on le crée
		if($cheminVerif != "" && !is_dir($cheminVerif))
		{
			$noProbleme = $noProbleme . mkdir($cheminVerif);
		}
	}
	
	//on va vérifier si le dernier élément est un fichier ou un répertoire. Pour cela on va chercher s'il existe un extension
	//si c'est un fichier et qu'il n'exite pas, on le crée vide
	//si c'est un répertoire et qu'il n'existe pas, on le crée
	$isRep = explode(".", $listeRep[$nb-1]);
	if(count($isRep)>1)
	{
		//si le fichier n'existe pas, on le crée
		if ($creation && !file_exists($chemin))
		{
		    $fichierBf = fopen($chemin, 'w+');
		    fwrite($fichierBf,"");
		    fclose($fichierBf);
		}
	}
	else
	{
		//si le répertoire à vérifier n'existe pas, on le crée
		if(!is_dir($chemin))
		{
			mkdir($chemin);
		}
	}	
}


/**
 * fonction qui crée le fichier .access dans le répertoire s'il n'existe pas encore
 * @param $chemin   Strong	: chemin du répertoire dans lequel on doit vérifier le fichier .access. Ce chemin peut être un chemin d'un fichier, dans ce cas, on regardera dans le répertoire contenant ce fichier
 * @param $htaccess	Strong	: contenu du fichier .access
 */
function verifHtaccess($chemin, $htaccess="deny from all\n")
{
	//on s'assure que tous les sous-répertoires existent
	verifChemin($chemin);

	//si le chemin n'indique pas un répertoire, on ne garde que le réptoire contenant le fichier
	if(!is_dir($chemin))
	{
		$pos=strrpos ($chemin, "/");
		$chemin = substr($chemin, 0, $pos);
	}

	//on construit le chemin vers le fichiers .htaccess : on ajoute un "/" a la fin du répertoire s'il n'y en a pas et si ce n'est pas le répertoire conurant
	if($chemin == "" || $chemin == "/")
	{
		$fileHtaccess = ".htaccess";
	}
	else
	{
		$fileHtaccess = $chemin . (strrpos($chemin, "/") + 1 == strlen($chemin) ? "" : "/") . ".htaccess";
	}

	//s'il n'y a pas encore de fichier .htaccess, on le crée
	if(!file_exists($fileHtaccess))
	{
		$fichier = @fopen($fileHtaccess, 'w+');
		@fwrite($fichier, $htaccess);
		@fclose($fichier);
	}
}



/**
 * fonction qui retourne la date de la news (format jj/mm/aaaa hh:mm:ss) au format RFC 2822
 * @param $date	String
 * @return String
 */
function dateRss($date)
{
	$annee = substr($date, 6, 4);
	$mois = substr($date, 3, 2);
	$jour = substr($date, 0, 2);
	$heure = substr($date, 11, 2);
	$minute = substr($date, 14, 2);
	$seconde = substr($date, 17, 2);

	return date("r", mktime($heure, $minute, $seconde, $mois, $jour, $annee));
}


/**
 * fonction qui se retourne le hash d'un texte
 * @param $txt	String
 * @return String
 */
function hashSB($txt)
{
	return hash('sha256', $txt);
}


/**
 * fonction de suppression récursive pour supprimer un repertoire et son contenu
 * @param $src	String : repretoire ou fichier à supprimer
 * @return Boolean : vrai si la suppression s'est effectuée dans la totalité
 */
function rdelete($src)
{
	$noProblem = true;
	
	//si c'est un répertoire, on doit d'abord le vider avant de le supprimer
	if (is_dir($src))
	{
		//on va parcourir tout le répertoire pour supprimer un à un tous les répertoires et fichiere
		$files = scandir($src);
		foreach ($files as $file)
		{
			//dans la liste, on aura forcément "." et "..", on va bien sur les ignorer
			if ($file != "." && $file != "..")
			{
				$noProblem = $noProblem && rdelete("$src/$file");
			}
		}
		$noProblem = $noProblem && rmdir($src);
	}
	
	//si c'est un fichier et qu'il existe, on le supprime
	else if (file_exists($src))
	{
		$noProblem = unlink($src);
	}

	return $noProblem;
}


/**
 * fonction qui ajoute un répertoire dans une archive zip
 * @param $zip	ZipArchive	: archive zip
 * @param $src	String		: chemin du répertoire à ajouter
 * @param $dst	String		: chemin à l'intérieur du zip
 * @return boolean	: vrai si tout c'est bien passé
 */
function addRepZip(ZipArchive &$zip, $src, $dst="")
{
	$noProblem = true;
	
	if (is_dir($src))
	{
		//on va copier chaque répertoire et fichier de manière récursive
		$files = scandir($src);
		foreach ($files as $file)
		{
			// dans la liste, on aura forcément "." et "..". On va bien sur les éviter
			if ($file != "." && $file != "..")
			{
				$noProblem = $noProblem && addRepZip($zip, ($src == "" ? "" : "$src/") . $file, ($dst == "" ? "" : "$dst/") . $file);
			}
		}
	}
	//si c'est un fichier et qu'il existe bien, on le copie dans le zip
	else if (file_exists($src))
	{
		$noProblem = $zip->addFile($src, $dst);
	}
	
	return $noProblem;
}


/**
 *fonciton qui génère une clé
 * @param $Texte	     String	: texte à crypter
 * @param $CleDEncryptage	String	: clé de cryptage
 * @return String : clé unique
 */
function generationCle($Texte,$CleDEncryptage)
{
	$CleDEncryptage = md5($CleDEncryptage);
	$Compteur=0;
	$VariableTemp = "";
	for ($Ctr=0;$Ctr<strlen($Texte);$Ctr++)
	{
		if ($Compteur==strlen($CleDEncryptage))
		$Compteur=0;
		$VariableTemp.= substr($Texte,$Ctr,1) ^ substr($CleDEncryptage,$Compteur,1);
		$Compteur++;
	}
	return $VariableTemp;
}


/**
 *fonction qui crypte un texte avec une clé
 * @param $Texte	     String	: texte à crypter
 * @param $Cle	   String	: clé de cryptage
 * @return String : $Texte crypté
 */
function crypte($Texte,$Cle)
{
	srand((double)microtime()*1000000);
	$CleDEncryptage = md5(rand(0,32000) );
	$Compteur=0;
	$VariableTemp = "";
	for ($Ctr=0;$Ctr<strlen($Texte);$Ctr++)
	{
		if ($Compteur==strlen($CleDEncryptage))
		$Compteur=0;
		$VariableTemp.= substr($CleDEncryptage,$Compteur,1).(substr($Texte,$Ctr,1) ^ substr($CleDEncryptage,$Compteur,1) );
		$Compteur++;
	}
	return base64_encode(generationCle($VariableTemp,$Cle) );
}


/**
 *fonction qui décrypte un texte avec une clé
 * @param $Texte	     String	: texte à décrypter
 * @param $Cle	   String	: clé de décryptage
 * @return String : $Texte décrypté
 */
function decrypte($Texte,$Cle)
{
	$Texte = generationCle(base64_decode($Texte),$Cle);
	$VariableTemp = "";
	for ($Ctr=0;$Ctr<strlen($Texte);$Ctr++)
	{
		$md5 = substr($Texte,$Ctr,1);
		$Ctr++;
		$VariableTemp.= (substr($Texte,$Ctr,1) ^ $md5);
	}
	return $VariableTemp;
}


/**
 *	fonction htmlentities avec les bon paramètre utf8
 * @param $texte	String
 * @return 	   String
 *	 
 */ 
function htmlentitiesUTF8($texte)
{
	return htmlentities($texte, ENT_COMPAT | ENT_HTML5, "UTF-8");
}



/**
 * fonction qui transforme une url en lien clickable
 * @param $texte	String
 * @param $target	String
 * @return String
 */
function lienClickable($texte, $target="_blank")
{
    return preg_replace('!(((?:https?|ftp)://)\S+[[:alnum:]]/?)!si','<a href="$1"' . ($target != "" ? " target='$target'" : "") . '>$1</a>',$texte);
}


/**
 * fonction qui supprime les accents d'une chaine de caractère
 * @param $string		String	: chaine dont on veut supprimer les accents
 * @return String : chaine sans les accents
 */

function supprimeAccents($str, $charset='utf-8')
{
    $str = htmlentities($str, ENT_NOQUOTES, $charset);
    
    $str = preg_replace('#&([A-za-z])(?:acute|cedil|caron|circ|grave|orn|ring|slash|th|tilde|uml);#', '\1', $str);
    $str = preg_replace('#&([A-za-z]{2})(?:lig);#', '\1', $str); // pour les ligatures e.g. '&oelig;'
    $str = preg_replace('#&[^;]+;#', '', $str); // supprime les autres caractères
    
    return $str;
}
?>
