<?php
/**
 * -----------------------------------------------
 * ------------------- Sali ----------------------
 * -----------------------------------------------
 *
 *  Script qui va gérer un liste de lien.
 *
 *  Les liens sont stockés dans un fichier.
 *
 *  On peut ajouter un lien directement depuis la page des liens ou via une popup accessible grace à du code javascript lancé comme un marque page.
 *  Ce raccourci se trouve dans la formulaire d'ajout de lien.
 *
 *  On peut ajouter, modifier et supprimer une catégorie (suppression uniquement si la catégorie ne contient pas de lien)
 *
 *  On peut modifier, supprimer un lien. Modifier = éditer l'url, le titre, la catégorie ou la position
 *
 *
 * Copyright 2012-2013 Superbaillot
 * This file is part of Sali.
 *
 *  Sali is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Sali is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Sali.  If not, see <http://www.gnu.org/licenses/>.
 */

session_start();
//importation du fichier de configuration du site
require_once("Inc/saliParametre.php");

//pour éviter un warning avec la fonction mktime
date_default_timezone_set($config->getTimezone());

define("VERSION_SALI", "4.0");							//version de sali
$alert = false;											//sert à savoir si on affiche le message d'info dans un div ou en alert javascript
$connexion = new Connexion($langue, NOM_SESSION_SALI);	//sert à gérer la connexion d'un utilisateur
$formulaireAjoutOuvert = false;							//défini s'il faut afficher ou non le formulaire d'ajout d'un nouveau lien. (en cas d'erreur de saisi, il faut le réafficher)
$filtreRecherche = new RechercheSali();					//le filtre de recherche est mémorisé en variable de session
$id = 0;												//id du lien ajouter dans la popup pour pouvoir l'éditer
$idDetail = 0;											//id du lien dont on veut afficher le détail
$idLienModifier = 0;									//id du lien à modifier. S'il est différent de 0, on va ouvrir la partie modification du lien en question dans la liste
$msg = "";												//message d'erreur ou d'information à afficher
$page = PAGE_LISTE;										//page à afficher (page liste, tag, rss, ou configuration, ceci n'est pas la pagination)
$pagination = 1;										//ceci est la pagination
$premiereConnexion = false;								//sert à savoir si c'est la première connexion pour forcer l'affichage de la page de configuration
$prive = $config->getPrive();							//privé coché par défaut ou non lors de l'ajout
$sali = new Sali(FICHIER_LISTES);						//liste des liens

$tags = "";												//tags du lien à ajouter avec le popup
$titre = "";											//titre du lien à ajouter avec le popup
$url = "";												//url du lien à ajouter avec le popup
$commentaire = "";										//description du lien à ajouter avec le popup
$visible = false;										//true = affichage du formulaire d'ajout de nouveau lien (true uniquement dans la popup d'ajout)

//première instance de Sali : on affiche la configuration avec un message d'info
if($config->getPassCrypte() == "" || $config->getPassCrypte() == hashSB(""))
{
	$premiereConnexion = true;
	$_POST['password'] = "";
	$page = PAGE_CONFIG;
	$msg = $langue->get("MSG_NO_PSWRD");
}

//si l'utilisateur veut changer de page
if(!empty($_GET['page']) && !$premiereConnexion)
{
	if(is_numeric($_GET['page'])&& $_GET['page'] >0 && $_GET['page']<= MAX_PAGE_SALI)
	{
		$page = $_GET['page'];
		if(!$connexion->estConnecte())
		{
			//vérification des pages dont un utilisateur non connecté à le droit de visité
			if(!in_array($page, array(PAGE_APROPOS, PAGE_DETAIL, PAGE_LISTE, PAGE_RSS, PAGE_TAG, PAGE_IMAGES)))
			{
				$page = PAGE_LISTE;
				$msg = $langue->get("MSG_PAGE_INEXISTANTE");
			}
		}
	}
	else
	{
		$msg = $langue->get("MSG_PAGE_INEXISTANTE");
	}
}

//si on a sauvegardé une pagination
if(isset($_SESSION['paginationSali']))
{
	$pagination = $_SESSION['paginationSali'];
}

//si on avait un filtre en court on le recharge
if(isset($_SESSION["filtreSali"]))
{
	$filtreRecherche = unserialize($_SESSION["filtreSali"]);
}

//si on clique sur le lien "liste des liens", je fais un reset des filtres et pagination pour afficher la liste par défaut
if((empty($_GET) && empty($_POST)) || (!empty($_GET['page']) && $_GET['page'] == PAGE_IMAGES))
{
	$filtreRecherche = new RechercheSali();
	$pagination = 1;
}

//on vérifie si on doit se connecter (utile pour pouvoir ajouter/modifier/supprimer des liens)
$msg .= $connexion->gererConnexion($config->getPassCrypte());

//je charge la liste maintenant que je sais si on est admin ou non
$sali->charger();

// on vérifie si on doit afficher la popup pour ajouter une url à la liste
if(isset($_GET['popup']) && $_GET['popup'] == "true")
{
	$page = PAGE_POPUP_AJOUT_LIEN;
	if(isset($_GET['url']))
	{
		$url = trim(stripslashes($_GET['url']));
	}
	if(isset($_GET['titre']))
	{
		$titre = trim(stripslashes($_GET['titre']));
	}
}

//on vérifie si on a tenté d'ajouter un lien via la popup. Dans ce cas, soit on réussi et on raffiche le formulaire avec un message de confirmation
//soit on a au moins un champ qui n'est pas correct et on raffiche le formulaire tel qu'il a été saisi pour que l'utilisateur puisse corriger l'erreur.
if(isset($_POST['popup']) && $_POST['popup'] == "true")
{
	$page = PAGE_POPUP_AJOUT_LIEN;
	$url = trim(stripslashes($_POST['url']));
	$titre = trim(stripslashes($_POST['titre']));
	$commentaire = trim(stripslashes($_POST['commentaire']));
	$tags = trim(stripslashes($_POST['tags']));
	$id = $_POST['id'];
	$prive = empty($_POST['admin']);
}

//pagination
if(isset($_GET['pagination']) && is_numeric($_GET['pagination']))
{
	$pagination = $_GET['pagination'];
}

//si l'utilisateur veut accéder un à lien en particulier
if(isset($_GET['id']) && is_numeric($_GET['id']))
{
	$idDetail = $_GET['id'];
	$page = PAGE_DETAIL;
}

//affichage du flux RSS
if(isset($_GET['rss']))
{
	//pour l'instant, je ne gère pas les filtres dans le flux RSS
	$filtreRecherche = new RechercheSali();
	$pagination = 1;
	$page = PAGE_RSS;
}

//si l'utilisateur a filtré
if((isset($_POST['rechercheMots']) && isset($_POST['rechercheTags'])) || isset($_GET['tag']))
{
	$rechercheTag = (isset($_POST['rechercheTags']) ? $_POST['rechercheTags'] : $_GET['tag']);
	$rechercheMot = (isset($_POST['rechercheMots']) ? $_POST['rechercheMots'] : "");

	//reinitialisation du filtre dans le cas ou il n'était pas vide car stocké en variable de session
	$filtreRecherche->init();

	$filtreRecherche->ajouteMot($rechercheMot);
	$filtreRecherche->ajouteTag($rechercheTag);
	
	$pagination=1;
}

//Si on est connecté, on peut effectuer des actions
if($connexion->estConnecte())
{
	//si c'est la première fois qu'on charge la page en étant connecté, je vérifie si une nouvelle version est disponible
	if($config->getCheckMaj() && !isset($_SESSION['premiereConnexionSali']))
	{
		//pour ne plus faire cette vérification
		$_SESSION['premiereConnexionSali'] = true;

		//on va chercher le numéro de la dernière version
		$handle = file(URL_CHECK_MAJ);
		$version = trim($handle[0]);

		if($version != "" && $version != VERSION_SALI)
		{
			$msg .= "<a href='" . URL_SALI_ARCHIVE . "'>" . $langue->get("MSG_NOUVELLE_VERSION", $version) . "</a><br>";
		}
	}

	//si on doit supprimer un lien
	if(isset($_POST['id_a_supprimer']) && is_numeric($_POST['id_a_supprimer']) && $_POST['id_a_supprimer']>0)
	{
		$alert = true;
		if($sali->supprimerLien($_POST['id_a_supprimer']))
		{
			$msg = $langue->get("MSG_LIEN_SUPPRIMER");
		}
		else
		{
			$msg = $langue->get("MSG_LIEN_INEXISTANT");
		}
	}

	//si on doit ajouter un lien
	else if(isset($_POST['url']))
	{
		$url = trim(stripslashes($_POST['url']));
		$titre = trim(stripslashes($_POST['titre']));
		if($titre == "")
		{
			$titre = $url;
		}
		$commentaire = trim(stripslashes($_POST['commentaire']));
		$tags = stripslashes($_POST['tags']);
		$prive = !empty($_POST['admin']);
		if(isset($_POST['id']) && is_numeric($_POST['id']))
		{
			$id = $_POST['id'];
		}
			
		//si l'url et le titre sont vide, on n'ajoute pas le lien
		//si l'url est vide, c'est une note
		//si le titre est vide, l'url servira de titre
		if($url == "" && $titre == "")
		{
			$msg = $langue->get("MSG_AJOUT_VIDE");
			$formulaireAjoutOuvert = true;
		}
		else
		{
			//lors de l'ajout via la popup, on réaffiche le formulaire rempli ave un méssage de confirmation de l'ajout.
			//si l'utilisateur change des informations et revalide, il faut faire une édition et plus un ajout
			$edit = ($id > 0);
			
			//si on vient d'ajouter un lien et qu'on le raffiche pour pouvoir l'éditer dans la popup
			if($edit)
			{
				$lien = $sali->getLienById($id);
				
				$lien->setUrl($url);
				$lien->setTitre($titre);
				$lien->setCommentaire($commentaire);
				$lien->ajouteTag($tags);
				$lien->setEtat($prive ? ETAT_VISIBLE_ADMIN : ETAT_VISIBLE_TOUS);

				//on récupère le favicon si on l'a configurer ainsi
				if($config->getRecupereFavicon())
				{
					$lien->recupreFavicon();
				}
				
				$sali->enregistrer();
			}
			else
			{
				//on ajoute le nouveau lien
				$lien = new Lien();
				
				$lien->setUrl($url);
				$lien->setTitre($titre);
				$lien->setCommentaire($commentaire);
				$lien->ajouteTag($tags);
				$lien->setEtat($prive ? ETAT_VISIBLE_ADMIN : ETAT_VISIBLE_TOUS);
				
				$lien = $sali->ajouterLien($lien);
				
				//le formulaire est près pour un nouvel ajout si on n'est pas dans la popup
				if($page != PAGE_POPUP_AJOUT_LIEN)
				{
					$url = "";
					$titre = "";
					$commentaire = "";
					$tags = "";
					$prive = true;
				}
				$id = $lien->getId();
			}
			
			$pagination=1;

			if(!$edit)
			{
				$msg = $langue->get("MSG_LIEN_AJOUTE");
			}
			else
			{
				$msg = $langue->get("MSG_LIEN_EDITE");
			}
		}
	}

	//si on doit noter un lien comme privé
	else if(isset($_POST['checkAdmin']) && is_numeric($_POST['checkAdmin']) && $_POST['checkAdmin'] > 0)
	{
		$alert = true;
		//on va chercher le lien dont il faut changer la visiblité
		$lien = $sali->getLienById($_POST['checkAdmin']);
		
		if($lien->getEtat() == ETAT_VISIBLE_TOUS)
		{
			$lien->setEtat(ETAT_VISIBLE_ADMIN);
		}
		elseif($lien->getEtat() == ETAT_VISIBLE_ADMIN)
		{
			$lien->setEtat(ETAT_VISIBLE_TOUS);
		}

		$sali->enregistrer();
		
		$msg = $langue->get("MSG_LIEN_EDITE");;
	}

	//si on doit afficher la version éditable d'un lien
	else if(isset($_POST['lienAModifer']) && is_numeric($_POST['lienAModifer']) && $_POST['lienAModifer'] > 0)
	{
		$idLienModifier = $_POST['lienAModifer'];
	}

	//si on doit editer un lien
	else if(isset($_POST['id']) && isset($_POST['urlEdit']) && isset($_POST['titreEdit']) && isset($_POST['commentaireEdit'])
			 && isset($_POST['tagsEdit']))
	{
		$alert = true;
		//on vérifie que l'url et le titre ne sont pas vide
		if($_POST['urlEdit'] == "" && $_POST['titreEdit'] == "")
		{
			$msg = $langue->get("MSG_AJOUT_VIDE");
		}
		else
		{
			//on va chercher le lien qu'il faut éditer
			$lien = $sali->getLienById($_POST['id']);
			
			//si le lien existe
			if($lien->getId() > 0)
			{
				$lien->setUrl($_POST['urlEdit']);
				$lien->setTitre(stripslashes($_POST['titreEdit']));
				$lien->setCommentaire(stripslashes($_POST['commentaireEdit']));
				$lien->setEtat(!empty($_POST['adminEdit'])? ETAT_VISIBLE_ADMIN : ETAT_VISIBLE_TOUS);
					
				if(isset($_POST['tagsEdit']))
				{
					$lien->setListeTag(array());
					$lien->ajouteTag($_POST['tagsEdit']);
				}

				//si on autorise la récupération des favicons, on le récupère
				if($config->getRecupereFavicon())
				{
					$lien->recupreFavicon();
				}
				//on sauvegarde
				$sali->enregistrer();

				$msg = $langue->get("MSG_LIEN_EDITE");
			}
			else
			{
				$msg = $langue->get("MSG_LIEN_INEXISTANT");
			}
		}
	}

	//export
	elseif(isset($_POST["export"]))
	{
		$sali->export();
	}

	//import
	elseif(isset($_FILES["import"]))
	{
		$msg = $sali->import();
		$page = PAGE_CONFIG;
	}

	//si l'utilisateur veut faire une sauvegarde
	elseif(isset($_POST['sauvegarde']) && $_POST['sauvegarde'] == "true")
	{
		getZipSauvegarde();
	}

	//si l'utilisateur veut charger un fichier sauvegardé
	elseif(isset($_FILES['chargement']))
	{
		getZipChargement();
		$page = PAGE_CONFIG;
	}

	//si on doit mettre à jour la configuration (dans l'absolu, je devrais tester tous les champs...)
	elseif(isset($_POST['pseudo']))
	{
		$alert = true;
		$msg = $config->maj();
	}
}

$_SESSION["filtreSali"] = serialize($filtreRecherche);
$_SESSION['paginationSali'] = $pagination;;

/************************************************     Génération de la page      **************************************************/

//FIXME
switch($page)
{
	case PAGE_LISTE :
		require("Inc/Vues/pageListeLien.php");
		break;
	case PAGE_POPUP_AJOUT_LIEN :
		require("Inc/Vues/pagePopup.php");
		break;
	case PAGE_DETAIL :
		require("Inc/Vues/pageDetail.php");
		break;
	case PAGE_CONFIG :
		require("Inc/Vues/pageConfiguration.php");
		break;
	case PAGE_APROPOS :
		require("Inc/Vues/pageAPropos.php");
		break;
	case PAGE_TAG :
		require("Inc/Vues/pageTag.php");
		break;
	case PAGE_RSS :
		require("Inc/Vues/xmlRss.php");
		break;
	case PAGE_IMAGES :
		require("Inc/Vues/pageImage.php");
		break;
}

/************************************************     fonctions diverses       **************************************************/

/**
 * fonction de tri des Lien du plus récent au plus ancien
 * @param $c1	Lien
 * @param $c2	Lien
 * @return int
 */
function triParDate(Lien   $l1, Lien $l2)
{
	if ($l1->getDate() == $l2->getDate())
	{
		return ($l1->getTitre() > $l2->getTitre()) ? 1 : -1;
	}
	return ($l1->getDate() < $l2->getDate()) ? 1 : -1;
}


/**
 * formate un timestamp au format type "dimanche 5 févirier 2011 à 12h48)
 * @param $time		int	: timestamp
 * @return String : la date au bon format (voir ci-dessus)
 */
function formatDate($time)
{
	$date = "";
	switch (date("N", $time))
	{
		case 1 : $date .= $GLOBALS["langue"]->get("LUNDI");
		break;
		case 2 : $date .= $GLOBALS["langue"]->get("MARDI");
		break;
		case 3 : $date .= $GLOBALS["langue"]->get("MERCREDI");
		break;
		case 4 : $date .= $GLOBALS["langue"]->get("JEUDI");
		break;
		case 5 : $date .= $GLOBALS["langue"]->get("VENDREDI");
		break;
		case 6 : $date .= $GLOBALS["langue"]->get("SAMEDI");
		break;
		case 7 : $date .= $GLOBALS["langue"]->get("DIMANCHE");
		break;
	}
	$date .= " " . date("d", $time) . " ";
	switch (date("n", $time))
	{
		case 1 : $date .= $GLOBALS["langue"]->get("JANVIER");
		break;
		case 2 : $date .= $GLOBALS["langue"]->get("FEVRIER");
		break;
		case 3 : $date .= $GLOBALS["langue"]->get("MARS");
		break;
		case 4 : $date .= $GLOBALS["langue"]->get("AVRIL");
		break;
		case 5 : $date .= $GLOBALS["langue"]->get("MAI");
		break;
		case 6 : $date .= $GLOBALS["langue"]->get("JUIN");
		break;
		case 7 : $date .= $GLOBALS["langue"]->get("JUILLET");
		break;
		case 8 : $date .= $GLOBALS["langue"]->get("AOUT");
		break;
		case 9 : $date .= $GLOBALS["langue"]->get("SEPTEMBRE");
		break;
		case 10 : $date .= $GLOBALS["langue"]->get("OCTOBRE");
		break;
		case 11 : $date .= $GLOBALS["langue"]->get("NOVEMBRE");
		break;
		case 12 : $date .= $GLOBALS["langue"]->get("DECEMBRE");
		break;
	}
	$date .= " " . date(" Y " . ("à") . " H\hi", $time) . " ";

	return $date;
}



/**
 * fonction qui retourne l'url du serveur avec le bon protocol
 * @return String
 */
function getServeur($avecScript=true)
{
	$https = (!empty($_SERVER['HTTPS']) && (strtolower($_SERVER['HTTPS'])=='on')) || $_SERVER["SERVER_PORT"]=='443'; // HTTPS detection.
	$serverport = ($_SERVER["SERVER_PORT"]=='80' || ($https && $_SERVER["SERVER_PORT"]=='443') ? '' : ':'.$_SERVER["SERVER_PORT"]);

	return "http" . ($https?"s":"") . "://" . $_SERVER['SERVER_NAME'].$serverport . ($avecScript ? $_SERVER['SCRIPT_NAME'] : pathinfo($_SERVER['SCRIPT_NAME'], PATHINFO_DIRNAME) . "/");
}


// Tells if a string start with a substring or not.
function startsWith($haystack,$needle,$case=true)
{
	if($case)
	{
		return (strcmp(substr($haystack, 0, strlen($needle)),$needle)===0);
	}
	return (strcasecmp(substr($haystack, 0, strlen($needle)),$needle)===0);
}


/**
 * fonction qui va faire un zip du répertoire Save et l'envoyer
 */
function getZipSauvegarde()
{
	//je crée l'archive dans Inc pour que le fichier temporaire ne soit pas public. (grâce au htaccess)
	$filename = "./Inc/sali.zip";
	if(file_exists($filename))
	{
		unlink($filename);
	}

	$zip = new ZipArchive();

	if ($zip->open($filename, ZIPARCHIVE::CREATE)!==true)
	{
		exit("Impossible d'ouvrir le fichier <$filename>\n");
	}

	addRepZip($zip, "./Save", "Save");

	$zip->close();

	$nomArchive = "Sali_" . date("Y-m-d") . ".zip";
	//on va envoyer le fichier
	header('Pragma: public');
	header('Cache-Control: must-revalidate, pre-check=0, post-check=0, max-age=0');
	header('Content-Tranfer-Encoding: none');
	header('Content-Length: ' . filesize($filename));
	header('Content-Type: application/octetstream; name="' . $nomArchive . '"');
	header('Content-Disposition: attachment; filename="' . $nomArchive . '"');
	header('Date: ' . gmdate(DATE_RFC1123));

	readfile($filename);

	//je supprime le fichier du serveur
	unlink($filename);

	exit; // nécessaire pour être certain de ne pas envoyer de fichier corrompu
}


/**
 * méthode qui charge un fichier de sauvegarde
 * @return String message d'info
 */
function getZipChargement()
{
	$msg = "";
	$filename = basename( $_FILES['chargement']['name']);
	if( move_uploaded_file($_FILES['chargement']['tmp_name'], $filename))
	{
		$zip = new ZipArchive;
		if ($zip->open($filename) === true)
		{
			if(is_dir(REPERTOIRE_SAUVEGARDE))
			{
				rdelete(REPERTOIRE_SAUVEGARDE);
			}
			$zip->extractTo('.');
				
			$zip->close();

			$GLOBALS['sali']->charger();
		}
		else
		{
			$msg .= $langue->get("MSG_NON_VALIDE") . "\n";
		}

		//le fichier à été lu jusqu'au bout, on peut le supprimer
		unlink($filename);
	}
	else
	{
		$msg .= $langue->get("MSG_NON_RECUPERE") . "\n";
	}
	return $msg;
}
?>
